var inputsStack = function(callback) {
    
    var stack = {};
    var factory = inputFactory;
    
    var add = function(data, value) {
        if (!getFactory().hasOwnProperty(data.type)) {
            console.warn('Theres no input component with type name' + data.type);
            return;
        }
        var input = factory[data.type](function() {
            // call the function like that to no lost the context passed to the function
            callback(api);
        }, data, value);
        stack[data.name] = {
            data: data,
            input: input
        };
        if (data.default) {
            input.setValue(data.default);
        }
        return input;
    };
    
    var reduce = function(f, init) {
        return Object.keys(getStack()).reduce(function(acc, key, i) {
            return f(acc, getStack()[key], i, getStack());
        }, init);
    };
    
    var setInputValues = function(settings) {
        Object.keys(settings).forEach(function(name) {
            if (!stack.hasOwnProperty(name)) {
                return false;
            }
            
            stack[name].input.setValue(settings[name]);
        });
    };
    
    var getStack = function() {
        return stack;
    };
    
    var getFactory = function() {
        return factory;
    };
    
    var api = {
        add: add,
        reduce: reduce,
        getStack: getStack,
        getFactory: getFactory,
        setInputValues: setInputValues
    };
    
    return api;
    
};

var inputsHelpers = (function() {
    
    return {
        execChange: function(container, type) {
            if (!container.onChange) {
                return console.warn('Container not found in '+ type +' input');
            }

            setTimeout(container.onChange);
        },
        
        getInterfaceSchema: function() {
            return Ext.getStore('Interfaces').findRecord('name', 'priax', false, true, true);
        },
        
        getIntegrationSelected: function() {
            return Ext.getStore('Integrations').data.items.reduce(function(_, integration) {
                return op.Maybe.Just(integration);
            }, op.Maybe.Nothing);
        }
    };
    
})();

var inputFactory = (function() {
    
    var text = function(change, data, value) {
        return Ext.create(
            Ext.apply({}, {
                xtype: 'textfield',
                fieldLabel: getLabel(data),
                emptyText: getDescription(data),
                value: value,
                listeners: {
                    change: change
                }
            }, commonParams)
        );
    };
    
    var textValidation = function(text) {
        return (typeof text === "string") && (text.length > 0);
    };
    
    var password = function(change, data, value) {
        return Ext.create(
            Ext.apply({}, {
                xtype: 'custominputs.password',
                fieldLabel: getLabel(data),
                emptyText: getDescription(data),
                value: value,
                onChange: change
            }, commonParams)
        );
    };
    
    var passwordValidation = textValidation;
    
    var boolean = function(change, data, value) {
        var Cmp = Ext.create(
            Ext.apply({}, {
                xtype: 'custominputs.boolean',
                fieldLabel: getLabel(data),
                value: value,
                rawValue: value,
                onChange: change
            }, commonParams)
        );
        return Cmp;
    };
    
    var select = function(change, data, value) {
        var store = op.Object.pluck('request', data.data)
            .fold(
                op.Functions.K(null),
                createSelectStore
            );
        if (value) {
            store.load(function() {
                input.setValue(value);
            });
        }
        var input = Ext.create(
            Ext.apply({}, {
                xtype: 'combobox',
                fieldLabel: getLabel(data),
                emptyText: getDescription(data),
                store: store,
                listeners: {
                    change: change
                },
                displayField: 'value',
                valueField: 'key',
                forceSelection: true
            }, commonParams)
        );
        return input;
    };
    
    var createSelectStore = function(action) {
        return Ext.create('Ext.data.Store', {
            proxy: {
                type: 'ajax',
                extraParams: {
                    schema: action.schema,
                    method: action.action
                },
                url: '/opmon/seagull/www/index.php/opinterface/action/callSchemaMethod',
                actionMethods: {
                    create : 'POST',
                    read   : 'POST',
                    update : 'POST',
                    destroy: 'POST'
                },
                reader: {
                    type: 'json',
                }
            }
        });
    };
    
    var selectValidation = function(value) {
        return value && value.length && value.length > 0;
    };
    
    var getLabel = function(data) {
        return data.label[getLanguage()];
    };
    
    var getDescription = (function(input) {
        return function(data) {
            input.innerHTML = data.description[getLanguage()];
            return input.value;
        };
    })(document.createElement('textarea'));
    
    var getLanguage = (function(lang) {
        return function() {
            if (!lang) {
                lang = parent.parent.parent.parent.getCookie('user_language');
            }
            return lang || 'en_US';
        };
    })();
    
    var commonParams = {
        flex: 1,
        width: '100%',
        labelWidth: 150,
        labelAlign: 'right',
        margin: '0 0 20 0'
    };
    
    return {
        text: text,
        textValidation: textValidation,
        password: password,
        passwordValidation: passwordValidation,
        boolean: boolean,
        select: select,
        selectValidation: selectValidation
    };
    
}) ();